
unsigned short  spuMem[256*1024];
unsigned char * spuMemC;
unsigned char * pSpuIrq=0;

unsigned short regArea[10000];                         // psx buffer

unsigned short spuCtrl, spuStat, spuIrq=0;             // some vars to store psx reg infos
unsigned long  spuAddr=0xffffffff;                     // address into spu mem

void (__stdcall *irqCallback)(void)=0;                 // func of main emu, called on spu irq

void (__stdcall *cddavCallback)(unsigned short,unsigned short)=0; // func of main emu, called to set cdda volume

////////////////////////////////////////////////////////////////////////
// NEW INTERFACE: write/read register & write/read dma & register irq 
////////////////////////////////////////////////////////////////////////

void CALLBACK SPUwriteRegister(unsigned long reg, unsigned short val)
{
 unsigned long r=reg&0xfff;
 regArea[(r-0xc00)>>1] = val;

 if(r>=0x0c00 && r<0x0d80)
  {
   int ch=(r>>4)-0xc0;
   switch(r&0x0f)
    {
     //------------------------------------------------// l volume
     case 0:                                           
       SPU_SetVolumeL(ch,val);
       return;
     //------------------------------------------------// r volume
     case 2:                                           
       SPU_SetVolumeR(ch,val);
       return;
     //------------------------------------------------// pitch
     case 4:                                           
       SPU_SetPitch(ch,val);
       return;
     //------------------------------------------------// start
     case 6:      
       SPU_SetStart(ch,val);
       return;
     //------------------------------------------------// level
     case 8:
       SPU_SetADSR1(ch,val);
       return;
     //------------------------------------------------// rate 
     case 10:
       SPU_SetADSR2(ch,val);
       return;
     //------------------------------------------------// adsr volume... mmm have to investigate this
     case 12:
       return;
     //------------------------------------------------//
     case 14:                                          // loop? ... mmm have to investigate this
       return;
     //------------------------------------------------//
    }
   return;
  }

 switch(r)
   {
    //-------------------------------------------------//
    case H_SPUaddr:
        spuAddr = (unsigned long) val<<3;
        return;
    //-------------------------------------------------//
    case H_SPUdata:
        spuMem[spuAddr>>1] = val;
        spuAddr+=2;
        if(spuAddr>0x7ffff) spuAddr=0;
        return;
    //-------------------------------------------------//
    case H_SPUctrl:
        spuCtrl=val;
        return;
    //-------------------------------------------------//
    case H_SPUstat:
        spuStat=val & 0xf800;
        return;
    //-------------------------------------------------//
    case H_SPUirqAddr:
        spuIrq = val;
        return;
    //-------------------------------------------------//
    case H_SPUon1:
        SoundOn(0,16,val);
        return;
    //-------------------------------------------------//
    case H_SPUon2:
        SoundOn(16,24,val);
        return;
    //-------------------------------------------------//
    case H_SPUoff1:
        SoundOff(0,16,val);
        return;
    //-------------------------------------------------//
    case H_SPUoff2:
        SoundOff(16,24,val);
        return;
    //-------------------------------------------------// dunno if that one works
    case H_CDLeft:
      if(cddavCallback) cddavCallback(0,val);
      return;
    case H_CDRight:
      if(cddavCallback) cddavCallback(1,val);
      return;
    //-------------------------------------------------//
    case H_FMod1:
        return;
    //-------------------------------------------------//
    case H_FMod2:
        return;
    //-------------------------------------------------//
    case H_Noise1:
        return;
    //-------------------------------------------------//
    case H_Noise2:
        return;
    //-------------------------------------------------//
    case H_RVBon1:
        ReverbOn(0,16,val);
        return;
    //-------------------------------------------------//
    case H_RVBon2:
        ReverbOn(16,24,val);
        return;
    //-------------------------------------------------//
    case H_Reverb:
        SetReverb(val);
        return;
   }
}

////////////////////////////////////////////////////////////////////////

unsigned short CALLBACK SPUreadRegister(unsigned long reg)
{
 switch(reg&0xfff)
  {
    case H_SPUctrl:
        return spuCtrl;

    case H_SPUstat:
        return spuStat;
        
    case H_SPUaddr:
        return (unsigned short)(spuAddr>>3);

    case H_SPUdata:
      {
       unsigned short s=spuMem[spuAddr>>1];
       spuAddr+=2;
       if(spuAddr>0x7ffff) spuAddr=0;
       return s;
      }

    case H_SPUirqAddr:
        return spuIrq;

  }
 return regArea[((reg&0xfff)-0xc00)>>1];
}
 
////////////////////////////////////////////////////////////////////////

unsigned short CALLBACK SPUreadDMA(void)
{
 unsigned short s=spuMem[spuAddr>>1];
 spuAddr+=2;
 if(spuAddr>0x7ffff) spuAddr=0;
 return s;
}

////////////////////////////////////////////////////////////////////////
  
void CALLBACK SPUwriteDMA(unsigned short val)
{
 spuMem[spuAddr>>1] = val;                             // spu addr got by writeregister
 spuAddr+=2;                                           // inc spu addr
 if(spuAddr>0x7ffff) spuAddr=0;                        // wrap
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUwriteDMAMem(unsigned short * pusPSXMem,int iSize)
{
 int i;
 for(i=0;i<iSize;i++)
  {
   spuMem[spuAddr>>1] = *pusPSXMem++;
   spuAddr+=2;                       
   if(spuAddr>0x7ffff) spuAddr=0;    
  }
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUreadDMAMem(unsigned short * pusPSXMem,int iSize)
{
 int i;
 for(i=0;i<iSize;i++)
  {
   *pusPSXMem++=spuMem[spuAddr>>1];  
   spuAddr+=2;                       
   if(spuAddr>0x7ffff) spuAddr=0;    
  }
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUregisterCallback(void (__stdcall *callback)(void))
{
 irqCallback = callback;
}

void CALLBACK SPUregisterCDDAVolume(void (__stdcall *CDDAVcallback)(unsigned short,unsigned short))
{
 cddavCallback = CDDAVcallback;
}

////////////////////////////////////////////////////////////////////////
// FREEZE
////////////////////////////////////////////////////////////////////////

typedef struct
{
 char          szSPUName[8];          // unique name of spu save state
 unsigned long ulFreezeVersion;       // version... always 1 right now
 unsigned long ulFreezeSize;          // sizeof(SPUFreeze_t) + trailing extra data
 unsigned char cSPUPort[0x200];       // spu registers
 unsigned char cSPURam[0x80000];      // spu ram
 xa_decode_t   xaS;                   // current xa block (if any)
} SPUFreeze_t;

typedef struct                        // own extra data... 
{
 unsigned long Future[256];

} SPUOSSFreeze_t;

////////////////////////////////////////////////////////////////////////

long CALLBACK SPUfreeze(unsigned long ulFreezeMode,SPUFreeze_t * pF)
{
 int i;

 if(!pF) return 0;                                     // no struct? bye

 if(ulFreezeMode)                                      // mode 2: get size infos
  {                                                    // mode 1: get spu state (saving)
   if(ulFreezeMode==1) 
    memset(pF,0,sizeof(SPUFreeze_t)+sizeof(SPUOSSFreeze_t));

   strcpy(pF->szSPUName,"PBOSS");                      // that's the unique name (here's the sign of my OSS spu :)
   pF->ulFreezeVersion=1;                              // set it always to 1 for now
   pF->ulFreezeSize=sizeof(SPUFreeze_t)+               // most important: set the size you need: sizeof(SPUFreeze_t)+sizeof(extra data)
                    sizeof(SPUOSSFreeze_t);

   if(ulFreezeMode==2) return 1;                       // ok, that's all in mode 2

   memcpy(pF->cSPURam,spuMem,0x80000);                 // in mode 1 we have to fill the spu infos
   memcpy(pF->cSPUPort,regArea,0x200);

   if(xapGlobal && XAPlay!=XAFeed)                     // here's my way to store active xa sound
    {
     pF->xaS=*xapGlobal;     
    }
   else 
    memset(&pF->xaS,0,sizeof(xa_decode_t));            // if you don't have xa infos: clear the struct

                                                       // ... here fill your extra data
                                                       // ... maybe adsr vals or whatever :)

   return 1;                                           // done :)
  }

 if(ulFreezeMode!=0) return 0;                         // now on to mode 0 (loading)

 memcpy(spuMem,pF->cSPURam,0x80000);                   // copy data from save state
 memcpy(regArea,pF->cSPUPort,0x200);

 if(pF->xaS.nsamples<=4032)                            // well, here we start xa playing again (if some xa data was stored)
  SPUplayADPCMchannel(&pF->xaS);


 // restore here you spu infos as needed.... check the
 // unique name, and get your extra datas again, and so on

 return 1;
}

////////////////////////////////////////////////////////////////////////
// COMMON INFO FUNCS
////////////////////////////////////////////////////////////////////////

char * CALLBACK PSEgetLibName(void)
{
 return libraryName;
}

////////////////////////////////////////////////////////////////////////

unsigned long CALLBACK PSEgetLibType(void)
{
 return PSE_LT_SPU;
}

////////////////////////////////////////////////////////////////////////

unsigned long CALLBACK PSEgetLibVersion(void)
{
 return version<<16|revision<<8|build;
}

////////////////////////////////////////////////////////////////////////
// XA AUDIO func... also used in new and old interface
////////////////////////////////////////////////////////////////////////

void CALLBACK SPUplayADPCMchannel(xa_decode_t *xap)
{
}

////////////////////////////////////////////////////////////////////////
// INIT/EXIT STUFF
////////////////////////////////////////////////////////////////////////

long CALLBACK SPUinit(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////

long CALLBACK SPUopen(HWND hW)                          
{
 return PSE_SPU_ERR_SUCCESS;        
}

////////////////////////////////////////////////////////////////////////

long CALLBACK SPUclose(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////

long CALLBACK SPUshutdown(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////
// MISC STUFF
////////////////////////////////////////////////////////////////////////

long CALLBACK SPUtest(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////

long CALLBACK SPUconfigure(void)
{
 return 0;
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUabout(void)
{
}

////////////////////////////////////////////////////////////////////////
// OLD INTERFACE... that's replaced with the funcs at the top of this file
////////////////////////////////////////////////////////////////////////

unsigned short CALLBACK SPUgetOne(unsigned long val)
{
 if(spuAddr!=0xffffffff)                               // hack, if emu
  {                                                    // is using getOne for
   return SPUreadDMA();                                // dma with new interface
  }

 if(val>=512*1024) val=512*1024-1;
 return spuMem[val/2];
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUputOne(unsigned long val,unsigned short data)
{
 if(spuAddr!=0xffffffff)                               // hack, if emu
  {                                                    // is using putOne for
   SPUwriteDMA(data);                                  // dma with new interface
   return;
  }

 if(val>=512*1024) val=512*1024-1;
 spuMem[ val/2 ] = data;
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUsetAddr(unsigned char ch, unsigned short waddr)
{
 SPU_SetStart(ch,waddr);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUsetPitch(unsigned char ch, unsigned short pitch)
{
 SetPitch(ch,pitch);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUsetVolumeL(unsigned char ch, short vol)
{
 SPU_SetVolumeL(ch,vol);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUsetVolumeR(unsigned char ch, short vol)
{
 SPU_SetVolumeR(ch,vol);
}               

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUstartChannels1(unsigned short channels)
{
 SoundOn(0,16,channels);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUstartChannels2(unsigned short channels)
{
 SoundOn(16,24,channels);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUstopChannels1(unsigned short channels)
{
 SoundOff(0,16,channels);
}

////////////////////////////////////////////////////////////////////////

void CALLBACK SPUstopChannels2(unsigned short channels)
{
 SoundOff(16,24,channels);
}

////////////////////////////////////////////////////////////////////////
// VERY OLD FUNC (PSEMU 0.00 eheh)... not supported by my spu
////////////////////////////////////////////////////////////////////////

void CALLBACK SPUplaySample(unsigned char ch)
{
}

